package net.minecraft.src;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.lwjgl.input.Keyboard;

import net.minecraft.client.Minecraft;

// SDK ModLoaderMp
@SuppressWarnings("rawtypes")
public class ModLoaderMp {
    public static final String NAME = "ModLoaderMP";
	
    private static boolean hasInit = false;
	private static Map<Integer, Class> netClientHandlerEntityMap = new HashMap<Integer, Class>();
	private static Map<Integer, BaseModMp> guiModMap = new HashMap<Integer, BaseModMp>();
	
	public static void Init()
	{
		if(!hasInit)
			init();
	}
	
	public static void HandleAllPackets(Packet200ModLoader packet200ModLoader)
	{
		if(!hasInit)
            init();
		
		if(packet200ModLoader.modId == NAME.hashCode())
			handleModCheck(packet200ModLoader);
		else
			for(int m = 0; m < ModLoader.getLoadedMods().size(); m++)
	        {
				BaseMod baseMod = (BaseMod)ModLoader.getLoadedMods().get(m);
				if(baseMod instanceof BaseModMp)
				{
					BaseModMp baseModMultiplayer = (BaseModMp)baseMod;
					if (baseModMultiplayer.getId() == packet200ModLoader.modId)
		            {
						baseModMultiplayer.HandlePacket(packet200ModLoader);
		            	break;
		            }
				}
	        }
	}

	public static Class HandleNetClientHandlerEntities(int id)
	{
		if(!hasInit)
            init();
        
		if(netClientHandlerEntityMap.containsKey(id))
			return netClientHandlerEntityMap.get(id);
		else
	        return null;
	}
	
	public static void SendPacket(BaseModMp baseModMp, Packet200ModLoader packet200ModLoader)
	{
		if(!hasInit)
            init();
        
		if(baseModMp == null)
		{
			IllegalArgumentException ex = new IllegalArgumentException("baseModMp cannot be null.");
			ModLoader.getLogger().throwing("ModLoaderMp", "SendPacket", ex);
            ModLoader.ThrowException("baseModMp cannot be null.", ex);
            return;
		}
		
		packet200ModLoader.modId = baseModMp.getId();
		sendPacket(packet200ModLoader);
	}
	
	public static void RegisterGUI(BaseModMp baseModMp, int inventoryType)
	{
		if(!hasInit)
            init();
		
		if(guiModMap.containsKey(inventoryType))
			System.out.println("RegisterGUI error: inventoryType already registered.");
		else
			guiModMap.put(inventoryType, baseModMp);
	}
	
	public static void HandleGUI(Packet100 packet100)
	{
		if(!hasInit)
            init();
		
		BaseModMp guiClass = guiModMap.get(packet100.inventoryType);
		GuiScreen guiScreen = guiClass.HandleGUI(packet100.inventoryType);
		if(guiScreen != null)
		{
			ModLoader.OpenGUI(ModLoader.getMinecraftInstance().thePlayer, guiScreen);
			ModLoader.getMinecraftInstance().thePlayer.craftingInventory.windowId = packet100.windowId;
		}
	}
	
	public static void RegisterNetClientHandlerEntity(Class entityClass, int entityId)
	{
		if(!hasInit)
            init();
		
		if(entityId > 255)
			System.out.println("RegisterNetClientHandlerEntity error: entityId cannot be greater than 255.");
		else if(netClientHandlerEntityMap.containsKey(entityId))
			System.out.println("RegisterNetClientHandlerEntity error: entityId already registered.");
		else
		{
			if(entityId > 127)
				entityId -= 256;
			
			netClientHandlerEntityMap.put(entityId, entityClass);
		}
	}
	
	/* Don't send the actual key, their key bindings may be different. Just
	 * pick a value for each key.                                            */
	public static void SendKey(BaseModMp baseModMp, int keyCode)
	{
		if(!hasInit)
            init();
		
		if(baseModMp == null)
		{
			IllegalArgumentException ex = new IllegalArgumentException("baseModMp cannot be null.");
			ModLoader.getLogger().throwing("ModLoaderMp", "SendKey", ex);
            ModLoader.ThrowException("baseModMp cannot be null.", ex);
            return;
		}
		
		Packet200ModLoader packetModCheckResponse = new Packet200ModLoader();
		packetModCheckResponse.modId = NAME.hashCode();
		packetModCheckResponse.packetType = 1;
		packetModCheckResponse.dataInt = new int[] { baseModMp.getId(), keyCode };
		
		sendPacket(packetModCheckResponse);
	}
    
	private static void init()
    {
    	hasInit = true;
    	
    	try
    	{
    		// Add custom packet mapping
    		Method method_AddIdClassMapping;
	        try
	        {
	            method_AddIdClassMapping = (Packet.class).getDeclaredMethod("a", new Class[] {
	            		int.class, Class.class
	            });
	        }
	        catch(NoSuchMethodException nosuchmethodexception)
	        {
	        	method_AddIdClassMapping = (Packet.class).getDeclaredMethod("addIdClassMapping", new Class[] {
	        			int.class, Class.class
	            });
	        }
	        method_AddIdClassMapping.setAccessible(true);
	        method_AddIdClassMapping.invoke(null, 200, Packet200ModLoader.class);
    	}
		catch (IllegalAccessException illegalaccessexception)
		{
			ModLoader.getLogger().throwing("ModLoaderMp", "init", illegalaccessexception);
			ModLoader.ThrowException("An impossible error has occurred!", illegalaccessexception);
			return;
		}
        catch(IllegalArgumentException illegalargumentexception)
        {
        	ModLoader.getLogger().throwing("ModLoaderMp", "init", illegalargumentexception);
        	ModLoader.ThrowException("An impossible error has occurred!", illegalargumentexception);
			return;
        }
		catch(InvocationTargetException invocationtargetexception)
		{
			ModLoader.getLogger().throwing("ModLoaderMp", "init", invocationtargetexception);
			ModLoader.ThrowException("An impossible error has occurred!", invocationtargetexception);
			return;
		}
        catch(NoSuchMethodException nosuchmethodexception)
        {
        	ModLoader.getLogger().throwing("ModLoaderMp", "init", nosuchmethodexception);
        	ModLoader.ThrowException("An impossible error has occurred!", nosuchmethodexception);
			return;
        }
    	catch(SecurityException securityexception)
        {
            ModLoader.getLogger().throwing("ModLoaderMp", "init", securityexception);
            ModLoader.ThrowException("An impossible error has occurred!", securityexception);
			return;
        }
    }
	
	private static void handleModCheck(Packet200ModLoader packet200ModLoader)
	{
		Packet200ModLoader packetModCheckResponse = new Packet200ModLoader();
		packetModCheckResponse.modId = NAME.hashCode();
		packetModCheckResponse.packetType = 0;
		
		packetModCheckResponse.dataString = new String[ModLoader.getLoadedMods().size()];
		for(int m = 0; m < ModLoader.getLoadedMods().size(); m++)
			packetModCheckResponse.dataString[m] = ((BaseMod)ModLoader.getLoadedMods().get(m)).toString();
		
		sendPacket(packetModCheckResponse);
	}
	
	private static void sendPacket(Packet200ModLoader packet200ModLoader)
	{
		if(ModLoader.getMinecraftInstance().theWorld != null && ModLoader.getMinecraftInstance().theWorld.multiplayerWorld)
			ModLoader.getMinecraftInstance().func_20001_q().addToSendQueue(packet200ModLoader);
	}
	
    private ModLoaderMp() { }
}
